@@ -1,11 +1,13 @@
 from _collections_abc import dict_keys, dict_values
 from pyg_base._as_list import as_list, as_tuple
+from pyg_base._as_float import as_float
 from pyg_base._dict import Dict
 from pyg_base._zip import zipper, lens
 from pyg_base._types import is_str, is_strs, is_arr, is_df, is_dicts, is_int, is_ints, is_tuple, is_bools, is_nan
 from pyg_base._tree import is_tree, tree_to_table
 from pyg_base._inspect import getargs
-from pyg_base._decorators import kwargs_support, try_none
+from pyg_base._decorators import kwargs_support, try_none, try_back
+from pyg_base._dates import ndt
 from pyg_base._sort import sort, cmp
 from pyg_base._encode import _encode, _obj
 from pyg_base._file import read_csv
@@ -16,6 +18,10 @@ import re
 __all__ = ['dict_concat', 'dictable', 'is_dictable']
 
 
+def nan2none(v):
+    return None if is_nan(v) or (is_str(v) and (len(v.strip()) == 0 or v[0] == '#' or v == 'n/a')) else v
+
+
 def dict_concat(*dicts):
     """
     A method of turning a list of dicts into one happy dict sharing its keys
@@ -93,7 +99,10 @@ def _data_columns_as_dict(data, columns = None):
         return dict_concat(tree_to_table(data, columns)) if is_tree(columns) else dict(data)
     if columns is not None:
         if is_str(columns):
-            return {columns : data}
+            if (is_str(data) and '.xls' in data) or isinstance(data, pd.io.excel.ExcelFile):
+                return dict(dictable.read_excel(data, columns))
+            else:
+                return {columns : data}
         else:
             return dict(zipper(columns, zipper(*data)))
     else:
@@ -101,7 +110,7 @@ def _data_columns_as_dict(data, columns = None):
             if data.endswith('.csv'):
                 data = read_csv(data)
             elif 'xls' in data:
-                data = pd.ExcelFile(data).parse()
+                data = dict(dictable.read_excel(data))
             else:
                 return dict(data = data)
         if is_df(data):
@@ -253,7 +262,7 @@ class dictable(Dict):
         kwargs = {key :_value(value) for key, value in kwargs.items()}
         data_kwargs = {key: _value(value) for key, value in _data_columns_as_dict(data, columns).items()}
         kwargs.update(data_kwargs)
-        if is_strs(columns) and not is_tree(columns):
+        if is_strs(columns) and (len(data_kwargs) == 0 or not is_str(columns)):
             kwargs = {key : kwargs.get(key, [None]) for key in columns} if len(kwargs)>0 else {key : [] for key in columns}
         n = lens(*kwargs.values())
         kwargs = {str(key) if is_int(key) else key : value * n if len(value)==1 else value for key, value in kwargs.items()}
@@ -494,6 +503,62 @@ class dictable(Dict):
             return type(self)([], self.keys())
         return res                
 
+
+    def if_none(self, none = None, **kwargs):
+        """
+        runs a column calculation if a column is not there, or if the existing value is considered missing.
+        
+        Example:
+        --------
+        >>> rs = dictable(a = [1,'hello'])
+        >>> rs = rs(b = try_none(lambda a: a[0]))  ## fails for numerical value
+        >>> rs
+
+        >>> dictable[2 x 2]
+        >>> a    |b   
+        >>> 1    |None
+        >>> hello|h   
+        
+        what shall we do with integer values for which b is None?
+        
+        >>> rs = rs.if_none(b = lambda a: a+3)
+        >>> rs
+        >>> dictable[2 x 2]
+        >>> a    |b   
+        >>> 1    |4
+        >>> hello|h   
+
+
+        Example:
+        -------- 
+        >>> rs = dictable(a = range(3))
+        >>> rs = rs(b = try_nan(lambda a: 10/a))
+        >>> rs = rs.if_none(np.nan, b = 'sorry no can do')
+
+        >>> dictable[3 x 2]
+        >>> a|b              
+        >>> 0|sorry no can do
+        >>> 1|10.0           
+        >>> 2|5.0        
+        
+        """
+        if none is None:
+            check = lambda value: value is None
+        elif not callable(none):
+            check = lambda value: value in as_list(none)
+        else:
+            check = none
+        res = self
+        for key, value in kwargs.items():
+            if key not in res.keys():
+                res[key] = res[value]
+            else:
+                if callable(value):
+                    res[key] = [row[key] if not check(row[key]) else row[value] for row in res]
+                else:
+                    res[key] = [row[key] if not check(row[key]) else value for row in res]                    
+        return res
+
     
     def apply(self, function):
         f = kwargs_support(function)
@@ -694,6 +759,32 @@ class dictable(Dict):
         rtn.update({k: [[self[k][i] for i in y] for y in ids] for k in self.keys() if k not in by})
         return rtn
     
+    @classmethod
+    def read_excel(cls, io, sheet_name = None, floats = None, ints = None, dates = None, no_none = None):
+        if is_str(io):
+            io = io.lower()
+            if '!' in io and sheet_name is None: ## handling input like: ExcelFile.xlsx!SheetName
+                io, sheet_name = io.split('!')
+            if not '.xls' in io:
+                io = io + '.xlsx'
+            io = pd.ExcelFile(io)
+        if sheet_name is None:
+            sheet_name = 0
+        df = io.parse(sheet_name)
+        res = cls(df)
+        res = res.relabel(lambda v: v.strip().replace(' ', '_'))
+        res = res.do(nan2none)
+        if floats:
+            res = res.do(try_back(as_float), floats)
+        if ints:
+            res = res.do(try_back(int), ints)
+        if dates:
+            res = res.do(try_back(ndt), dates)
+        if no_none:
+            for col in as_list(no_none):
+                res = res.exc(**{col : None})
+        return res
+    
     def unlist(self):
         """
         undoes listby...
@@ -810,6 +901,8 @@ class dictable(Dict):
         lcols = as_tuple(lcols); rcols = as_tuple(rcols)
         if len(lcols)!=len(rcols):
             raise ValueError('cannot inner join when cols on either side mismatch in length %s vs %s'%(lcols, rcols))
+        else:
+            print('inner joining on %s'%list(lcols))
         cols = []
         for lcol, rcol in zip(lcols, rcols):
             if is_str(lcol):
