@@ -24,7 +24,6 @@ import numpy as np
 from copy import copy
 import inspect
 import datetime
-from operator import add, mul
 
 
 __all__ = ['df_fillna', 'df_index', 'df_reindex', 'df_columns', 'presync', 'np_reindex', 'nona', 'df_slice', 'df_unslice', 'min_', 'max_', 'add_', 'mul_', 'sub_', 'div_', 'pow_']
@@ -470,19 +469,40 @@ def df_concat(objs, columns = None, axis=1, join = 'outer'):
     >>> 5  2.0  NaN
     >>> 6  3.0  NaN    
 
+    :Example: numpy arrays concatenation
+    -----------------------
+    >>> objs = [np.array([1,2,3]), np.array([4,5,6]), 3, 4]
+    >>> df_concat(objs)
+
     """
     if isinstance(objs, dict):
         columns = list(objs.keys())
         objs = list(objs.values())
-    if isinstance(objs, list):
+    if isinstance(objs, (list, tuple)):
         df_objs = [o for o in objs if is_pd(o)]
-        res = pd.concat(df_objs, axis = axis, join = join)
-        if len(df_objs) < len(objs):
-            df_objs = [o if is_pd(o) else pd.Series(o, res.index) for o in objs]
-            res = pd.concat(df_objs, axis = axis, join = join)            
+        np_objs = [o for o in objs if is_arr(o)]
+        if len(df_objs):
+            res = pd.concat(df_objs, axis = axis, join = join)
+            if len(df_objs) < len(objs):
+                df_objs = [o if is_pd(o) else pd.Series(o, res.index) for o in objs]
+                res = pd.concat(df_objs, axis = axis, join = join)
+        elif len(np_objs):
+            ns = set([o.shape[0] for o in np_objs])
+            if len(ns) == 1:
+                n = list(ns)[0]
+            else:
+                raise ValueError('numpy arrays must be of equal length, other joins not implemented')
+            np_objs = [(o[:, np.newaxis] if len(o.shape)==1 else o) if is_arr(o) else np.full((n,1), o) for o in objs]
+            res = np.concatenate(np_objs, axis = axis)
+        else:
+            return objs
     elif isinstance(objs, pd.DataFrame):
         res = objs.copy() if columns is not None else objs
-    if columns is not None:
+    elif is_arr(objs):
+        res = objs.copy()
+    else:
+        raise ValueError('not sure how to convert this into a single dataframe')
+    if columns is not None and is_df(res):
         if isinstance(columns, list):
             res.columns = columns 
         else:
@@ -875,6 +895,37 @@ def _pow_(a, b):
     return a**b
 
 
+@presync
+def _gt_(a, b):
+    """
+    comparison of a and b supporting presynching (inner join) of timeseries
+    """
+    return a > b
+
+@presync
+def _ge_(a, b):
+    """
+    comparison of a and b supporting presynching (inner join) of timeseries
+    """
+    return a >= b
+
+
+@presync
+def _lt_(a, b):
+    """
+    comparison of a and b supporting presynching (inner join) of timeseries
+    """
+    return a < b
+
+@presync
+def _le_(a, b):
+    """
+    comparison of a and b supporting presynching (inner join) of timeseries
+    """
+    return a <= b
+
+
+
 def add_(a, b = None, join = 'ij', method = None, columns = 'ij'):
     """
     a = pd.Series([1,2,3], drange(-2))
@@ -924,6 +975,30 @@ def pow_(a, b, join = 'ij', method = None, columns = 'ij'):
     """
     return _pow_(a,b, join = join, method = method, columns = columns)
 
+def gt_(a, b, join = 'ij', method = None, columns = 'ij'):
+    """
+    equivalent to a>b supporting presynching (inner join) of timeseries
+    """
+    return _gt_(a,b, join = join, method = method, columns = columns)
+
+def ge_(a, b, join = 'ij', method = None, columns = 'ij'):
+    """
+    equivalent to a>=b supporting presynching (inner join) of timeseries
+    """
+    return _ge_(a,b, join = join, method = method, columns = columns)
+
+def lt_(a, b, join = 'ij', method = None, columns = 'ij'):
+    """
+    equivalent to a<b supporting presynching (inner join) of timeseries
+    """
+    return _lt_(a,b, join = join, method = method, columns = columns)
+
+def le_(a, b, join = 'ij', method = None, columns = 'ij'):
+    """
+    equivalent to a<=b supporting presynching (inner join) of timeseries
+    """
+    return _le_(a,b, join = join, method = method, columns = columns)
+
 def min_(a, b = None, join = 'ij', method = None, columns = 'ij'):
     """
     equivalent to redced np.minimum operation supporting presynching of timeseries
@@ -1136,4 +1211,5 @@ def df_unslice(df, ub):
     res = res(ts = lambda lb, ub: df_slice(df, lb, ub, '(]'))
     res = res(rs = lambda i, ts: dictable(u = ub[i: i+n], j = range(len(ub[i: i+n])))(ts = lambda j: ts[j]))
     rs = dictable.concat(res.rs).listby('u').do([pd.concat, nona], 'ts')
-    return dict(rs['u', 'ts'])
\ No newline at end of file
+    return dict(rs['u', 'ts'])
+
